#include "keymap2"
/**********************************************
KBD is a char device: each register is a bytes
**********************************************/
#define KCNTL 0x00
#define KSTAT 0x04
#define KDATA 0x08
#define KCLK  0x0C
#define KISTA 0x10

typedef volatile struct kbd{
  char *base;     // base address as char *
  char buf[128];
  int head, tail, data, room;
}KBD;

KBD kbd;

int release;       // key release flag

int kbd_init()
{
    KBD *kp = &kbd;
    kp->base = (char *)0x10006000;

    *(kp->base + KCLK)  = 8;       // ARM manual says clock=8
    *(kp->base + KCNTL) = 0x02;    // control reg bit2 = KBD on

 A2: // write code to enable KBD interrupts (bit4=INTenable)
     // YOUR CODE here
  
    kp->head = kp->tail = 0;       // circular buffer char buf[128]
    kp->data = 0; kp->room = 128;

    release = 0;
}

void kbd_handler()
{
  u8 scode, c;
  KBD *kp = &kbd;

  color = YELLOW;

  scode = *(kp->base + KDATA);   // get scan code of this interrpt

  kputs("kbd interrupt scancode = "); kprintx(scode);

  if (scode == 0xF0){  // it's key release 
     release = 1;      // set release flag
     return;
  }

  if (release == 1){   // scan code after 0xF0
     release = 0;      // reset release flag
     return;
  }

  // map scode to ASCII in lowercase 
  c = ltab[scode];

  kputs("kbd interrupt : ");
  if (c != '\r')
    kputc(c);
  kputs("\n");

  kp->buf[kp->head++] = c;
  kp->head %= 128;
  kp->data++; kp->room--;
}

int kgetc()
{
  char c;
  int sr;
  KBD *kp = &kbd;

  unlock();                          // unmask IRQ in case it was masked out
  while(kp->data == 0);              // BUSY wait while kp->data is 0 

  sr = int_off();                   // mask out IRQ, sr=original cpsr
    c = kp->buf[kp->tail++]; 
    kp->tail %= 128;                 /*** Critical Region ***/
    kp->data--; kp->room++;
  int_on(sr);                      // restore original cpsr
    
  return c;
}

int kgets(char s[ ])
{
  char c;
  while( (c = kgetc()) != '\r'){
    *s = c;
    s++;
  }
  *s = 0;
}

